; Contributed by Bob Boyer
; watch.el
(require 'autorevert)
(require 'cl)
(defvar *watch-buffer* nil)
(defvar *watch-tmp-text* "~/watch-tmp.text")
(defvar *watch-tmp-lisp* "~/watch-tmp.lisp")
(defvar *watch-line-number* 0)
(defvar *watch-column-number* 0)

(set-face-font 'isearch
"-adobe-courier-bold-r-normal--16-*-*-*-*-*-*-*")

(defvar *color-alist* '(
    ("hons" . blue)
    ("count" . blue)
    ("size"  . blue)
    ("-ht" . blue)
    ("mht" .  dark\ green)
    ("calls from" . dark\ green)
    ("pons" . blue)
    ("memoize" . dark\ green)
    ("time" . red)
    ("calls=" . variable-pitch)
    ("seconds" . red)
    ; ("all outermost" . blue)
    ("gc" . default)
    ))

(defun watch ()
  (interactive "")
  (cond ((file-exists-p *watch-tmp-text*)
         (find-file *watch-tmp-text*))
        (*watch-buffer*
         (set-buffer *watch-buffer*)
         (kill-buffer nil)
         (find-file *watch-tmp-text*)))
  (setq *watch-buffer* (current-buffer))
  (goto-char 0)
  (setq *watch-line-number* 0)
  (setq *watch-column-number* 0)
  (color-buffer)
  (auto-revert-mode 1))

(defadvice ask-user-about-supersession-threat
  (before ask-user-about-supersession-threat-around)
  (cond ((eq (current-buffer) *watch-buffer*)
         (signal 'file-supersession (list "File reverted")))))

(ad-activate 'ask-user-about-supersession-threat)

(defadvice auto-revert-buffers
  (around revert-buffer-around)
  (let ((b (current-buffer)))
    (cond ((and (bufferp *watch-buffer*)
                (buffer-name *watch-buffer*))
           (set-buffer *watch-buffer*)
           (let ((p (point)))
             (beginning-of-line 1)
             (setq *watch-line-number*
                   (count-lines (point-min) (point)))
             (setq *watch-column-number* (- p (point)))
             (goto-char p))))
    (set-buffer b))
  ad-do-it
  (let ((b (current-buffer)))
    (cond ((and (bufferp *watch-buffer*)
                (buffer-name *watch-buffer*))
           (set-buffer *watch-buffer*)
           (color-buffer)
           (goto-line (1+ *watch-line-number*))
           (forward-char *watch-column-number*)))))

(ad-activate 'auto-revert-buffers)

(defun watch-lisp-eval-defun ()
  (interactive)
  (let ((p (point)))
    (end-of-line)
    (beginning-of-defun)
    (let ((beg (point)))
      (forward-sexp)
      (write-region beg (point) *watch-tmp-lisp*))
    (goto-char p)))

(defun watch-lisp-eval-region (start end)
  (interactive "r")
  (write-region start end *watch-tmp-lisp*))

(defun watch-acl2-eval-defun ()
  (interactive)
  (let ((p (point)))
    (end-of-line)
    (beginning-of-defun)
    (let ((beg (point)))
      (forward-sexp)
      (write-region "(ld '(" nil *watch-tmp-lisp*)
      (write-region beg (point) *watch-tmp-lisp* t)
      (write-region "))" nil *watch-tmp-lisp* t))
    (goto-char p)))

(defun color-buffer ()
  (interactive)
  (loop for pair in *color-alist* do
        (let ((d (cdr pair)))
          (unless (facep d)
            (copy-face 'default d)
            (set-face-foreground d (symbol-name d)))))
  (font-lock-mode 0)
  (let ((mod (buffer-modified-p nil))
        (p (point)))
    (remove-text-properties (point-min) (point-max) '(face))
    (loop for pair in *color-alist* do
          (goto-char (point-min))
          (loop while (search-forward (car pair) nil t)
                do
                (beginning-of-line 1)
                (let ((p (point)))
                  (end-of-line 1)
                  (put-text-property p (point) 'face (cdr pair)))))
    (goto-char p)
    (set-buffer-modified-p mod)))
  
(cond ((not (boundp 'ctl-t-keymap))
       (setq ctl-t-keymap (make-sparse-keymap))
       (define-key (current-global-map) "\C-T" ctl-t-keymap)))

(define-key ctl-t-keymap "2"    'watch-acl2-eval-defun)
(define-key ctl-t-keymap "3"    'watch-lisp-eval-defun)
(define-key ctl-t-keymap "4"    'watch-lisp-eval-region)
(define-key ctl-t-keymap "\C-w" 'watch)
